package com.nanyan.securitylink.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.nanyan.securitylink.service.TokenRouteMappingService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;
import java.nio.charset.StandardCharsets;

@Slf4j
@Component
public class FailoverGatewayFilterFactory extends AbstractGatewayFilterFactory<FailoverGatewayFilterFactory.Config> {

    private static final ThreadLocal<String> REQUEST_BODY_CACHE = new ThreadLocal<>();
    private final WebClient.Builder webClientBuilder;
    private final TokenRouteMappingService tokenRouteMappingService;

    public FailoverGatewayFilterFactory(WebClient.Builder webClientBuilder,
                                        TokenRouteMappingService tokenRouteMappingService) {
        super(Config.class);
        this.webClientBuilder = webClientBuilder;
        this.tokenRouteMappingService = tokenRouteMappingService;
    }

    @Override
    public GatewayFilter apply(Config config) {
        return (exchange, chain) -> {
            try {
                String authHeader = exchange.getRequest().getHeaders().getFirst(HttpHeaders.AUTHORIZATION);
                String token = authHeader != null && authHeader.startsWith("Bearer ")
                        ? authHeader.substring(7)
                        : null;
                String sourceUri = exchange.getRequest().getPath().value();

                if (token != null) {
                    return tokenRouteMappingService.findMatchingUriConfig(token, sourceUri)
                            .map(uriConfig -> {
                                return cacheRequestBody(exchange).flatMap(cachedBody -> {
                                    log.info("Found matching URI config for token: {}, sourceUri: {}", token, sourceUri);
                                    return tryRequest(
                                            uriConfig.getPrimaryUrl(),
                                            sourceUri,
                                            uriConfig.getPrimaryHost(),
                                            exchange,
                                            true
                                    ).onErrorResume(primaryError -> {
                                        log.error("Primary endpoint failed: {}, error: {}",
                                                uriConfig.getPrimaryUrl() + sourceUri,
                                                primaryError.getMessage());

                                        String targetUri = tokenRouteMappingService.resolveTargetUri(sourceUri, uriConfig);
                                        return tryRequest(
                                                uriConfig.getFallbackUrl(),
                                                targetUri,
                                                uriConfig.getFallbackHost(),
                                                exchange,
                                                false
                                        );
                                    });
                                });
                            })
                            .orElse(chain.filter(exchange));
                }

                return chain.filter(exchange);
            } finally {
                // 确保在请求处理完成后清理 ThreadLocal
                REQUEST_BODY_CACHE.remove();
            }
        };
    }

    private Mono<String> cacheRequestBody(ServerWebExchange exchange) {

        ServerHttpRequest request = exchange.getRequest();
        if (!requiresRequestBody(request.getMethod())) {
            return Mono.just("");
        }

        // 读取并缓存请求体
        return DataBufferUtils.join(request.getBody())
                .map(dataBuffer -> {
                    try {
                        byte[] bytes = new byte[dataBuffer.readableByteCount()];
                        dataBuffer.read(bytes);
                        String body = new String(bytes, StandardCharsets.UTF_8);
                        // 将请求体存储在 ThreadLocal 中
                        log.info("cache body:{}", body);
                        REQUEST_BODY_CACHE.set(body);
                        return body;
                    } finally {
                        DataBufferUtils.release(dataBuffer);
                    }
                })
                .defaultIfEmpty("");
    }

    private Mono<Void> tryRequest(String baseUrl, String uri, String host,
                                  ServerWebExchange exchange, boolean isSourceRequest) {
        String cachedBody = REQUEST_BODY_CACHE.get();
        ServerHttpRequest request = exchange.getRequest();
        String fullUrl = baseUrl + (baseUrl.endsWith("/") ? uri.substring(1) : uri);

        log.info("Trying request to: {}", fullUrl);
        log.info("Using cached request body: {}", cachedBody);

        WebClient.RequestBodySpec requestBodySpec = webClientBuilder.build()
                .method(request.getMethod())
                .uri(fullUrl + (request.getQueryParams().isEmpty() ? "" :
                        "?" + request.getQueryParams().toString().substring(1)));

        // 复制请求头
        requestBodySpec.headers(headers -> {
            headers.addAll(request.getHeaders());
            headers.set(HttpHeaders.HOST, host);
        });

        // 设置请求体
        if (requiresRequestBody(request.getMethod()) && cachedBody != null && !cachedBody.isEmpty()) {
            MediaType contentType = request.getHeaders().getContentType();
            if (contentType != null) {
                requestBodySpec.contentType(contentType);
            }
            requestBodySpec.body(BodyInserters.fromValue(cachedBody));
        }

        return requestBodySpec
                .exchange()
                .flatMap(clientResponse -> handleResponse(exchange, clientResponse, isSourceRequest));
    }

    private Mono<Void> handleResponse(ServerWebExchange exchange,
                                      org.springframework.web.reactive.function.client.ClientResponse clientResponse,
                                      boolean isSourceRequest) {
        HttpStatus responseStatus = clientResponse.statusCode();

        if (isSourceRequest) {
            return clientResponse.bodyToMono(String.class)
                    .flatMap(body -> {
                        log.info("Source request response: status={}, body={}", responseStatus, body);

                        if (responseStatus != HttpStatus.OK) {
                            return Mono.error(new RuntimeException(
                                    String.format("Source request HTTP status not 200: %s, body: %s",
                                            responseStatus.value(), body)));
                        }

                        try {
                            JSONObject jsonBody = JSON.parseObject(body);
                            Integer bodyStatus = jsonBody.getInteger("status");
                            if (bodyStatus == null || bodyStatus != 200) {
                                return Mono.error(new RuntimeException(
                                        String.format("Source request body status not 200: %s, body: %s",
                                                bodyStatus, body)));
                            }

                            exchange.getResponse().setStatusCode(responseStatus);
                            exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());
                            DataBuffer buffer = exchange.getResponse().bufferFactory()
                                    .wrap(body.getBytes(StandardCharsets.UTF_8));
                            return exchange.getResponse().writeWith(Mono.just(buffer));

                        } catch (Exception e) {
                            log.error("Error parsing response body: {}", e.getMessage());
                            return Mono.error(new RuntimeException(
                                    String.format("Failed to parse response body as JSON: %s", body)));
                        }
                    });
        }

        exchange.getResponse().setStatusCode(responseStatus);
        exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());

        return clientResponse.bodyToMono(DataBuffer.class)
                .flatMap(buffer -> exchange.getResponse().writeWith(Mono.just(buffer)))
                .onErrorResume(throwable -> {
                    log.info("Error handling response: {}", throwable.getMessage());
                    exchange.getResponse().setStatusCode(HttpStatus.INTERNAL_SERVER_ERROR);
                    String errorMessage = throwable.getMessage();
                    DataBuffer buffer = exchange.getResponse().bufferFactory()
                            .wrap(errorMessage.getBytes(StandardCharsets.UTF_8));
                    return exchange.getResponse().writeWith(Mono.just(buffer));
                });
    }

    private boolean requiresRequestBody(HttpMethod method) {
        return method == HttpMethod.POST ||
                method == HttpMethod.PUT ||
                method == HttpMethod.PATCH;
    }

    @Data
    public static class Config {
    }
}